#pragma once

#include "wimp.h"
#include "clickable.h"
#include "coordinateconverter.h"
#include "unassignable.h"
#include <string>
#include <list>
#include "wimp_window.h"

namespace rov2
{
class CIcon;

/*! Defines a class for a basic window */
class CWindow : public CClickable, public CCoordinateConverter, public CUnassignable
{
public:
  /*! Constructs a window with the given title 
  \param rksTitle The window title (which is copied internally)
  */
  CWindow(const std::string& rksTitle);

  /*! Destroys a window */
  ~CWindow();

  /*! Creates the window
  \param rklSize The window size (one coordinate is [0, 0] - the other is calculated from this)
  */
  void Create(const tLocation& rklSize);

  /*! Gets the window handle
  \return The window handle
  */
  inline const tWindowHandle Handle() const { return m_whHandle; }

  /*! Gets the window title
  \return the window title
  */
  inline const std::string Title() const { return m_caWindowTitle; }

  /*! Sets the new window title
  \param rksNewTitle The new window title
  */
  virtual void SetTitle(const std::string& rksNewTitle);

  /*! Redraws an area of the window
  \param rklMinimum The lower-left coordinate of the window to redraw
  \param rklMaximum The top-right coordinate of the window to redraw
  */
  virtual void RedrawArea(const tLocation& rklMinimum, const tLocation& rklMaximum) {};

  /*! Opens the window
  \param rkwobOpenBlock The window open block request
  */
  virtual void Open(const tWindowOpenBlock& rkwobOpenBlock);

  /*! Closes the window */
  virtual void Close() {};

  /*! Called when something is clicked on the window
  \param rkmcbClickBlock The mouse click block request
  \return True if the click was handled; otherwise control passes to the next item
  */
  virtual bool Clicked(const tMouseClickBlock& rkmcbClickBlock);

  /*! Registers an icon
  \param kpiIcon The icon to register
  */
  void RegisterIcon(const CIcon* kpiIcon);

  /*! Removes an icon from the list
  \param kpiIcon The icon to remove */
  void DeregisterIcon(const CIcon* kpiIcon);

  /*! Finds the icon based on the icon handle
  \param kihIcon The icon handle to search for
  \return The icon the handle refers to, or NULL if not found
  */
  const CIcon* FindIcon(const tIconHandle kihIcon);

protected:
  CWindow() {};

  /*! Called to change any of the window flags
  \param rwcbBlock The window creation block
  */
  virtual void Adorn(tWindowCreateBlock& rwcbBlock) {};

  /*! Called after the window is created */
  virtual void Created(void) {};

  /*! The copy of the window title */
  char m_caWindowTitle[256];

  /*! The window handle*/
  tWindowHandle m_whHandle;

  /*! The list of icons on this window */
  std::list<const CIcon*> m_lkpwIcons;
};

}